package com.xjrsoft.module.system.controller;

import cn.dev33.satoken.session.SaSession;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.jwt.JWT;
import cn.hutool.jwt.JWTUtil;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xjrsoft.common.annotation.XjrLog;
import com.xjrsoft.common.constant.GlobalConstant;
import com.xjrsoft.common.enums.EnabledMark;
import com.xjrsoft.common.model.result.R;
import com.xjrsoft.common.sms.SmsSender;
import com.xjrsoft.common.utils.RedisUtil;
import com.xjrsoft.config.KeyCloakConfig;
import com.xjrsoft.module.organization.entity.User;
import com.xjrsoft.module.organization.service.IUserService;
import com.xjrsoft.module.system.dto.*;
import com.xjrsoft.module.system.service.ILoginService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.StringUtils;
import org.keycloak.authorization.client.AuthzClient;
import org.keycloak.authorization.client.Configuration;
import org.keycloak.representations.AccessTokenResponse;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 登录控制器
 * </p>
 *
 * @author tzx
 * @since 2022-03-02
 */
@Api(tags = "登录模块")
@RestController
@RequestMapping(GlobalConstant.SYSTEM_MODULE_PREFIX)
@AllArgsConstructor
public class LoginController {

    private final ILoginService loginService;
    private final IUserService userService;

    private final RedisUtil redisUtil;

    private KeyCloakConfig keyCloakConfig;

    private SmsSender smsSender;

    @PostMapping("/login")
    @ApiOperation(value = "登录", notes = "传入账号:account,密码:password")
    @XjrLog(value = "账号密码登录成功")
    public R login(@RequestBody @Valid LoginDto dto) throws Exception {
        return R.ok(loginService.login(dto));
    }

    @PostMapping("/loginByCode")
    @ApiOperation(value = "登录", notes = "code")
    @XjrLog(value = "Code登录成功")
    public R loginByCode(@RequestBody @Valid LoginByCodeDto dto) throws Exception {
        return R.ok(loginService.loginByCode(dto));
    }

    @GetMapping("/imgcaptcha")
    @ApiOperation(value = "图形验证码",notes = "图形验证码")
    public R imgCaptcha(){
        return R.ok(loginService.imgCaptcha());
    }

    @PostMapping("/create-token")
    @ApiOperation(value = "创建token", notes = "传入账号:account,密码:password")
    @XjrLog(value = "账号密码登录成功")
    public R createToken(@RequestBody @Valid CreateTokenDto dto) {
        return R.ok(loginService.createToken(dto));
    }


    /**
     * 发送验证码
     */
    @GetMapping("/captcha")
    @XjrLog(value = "发送验证码")
    @ApiOperation(value = "发送验证码", notes = "传入账号:mobile")
    public R captcha(@RequestParam String mobile) {
        //生成六位数的字符串
        String code = RandomUtils.nextInt(100000, 999999) + StringPool.EMPTY;

        //华为短信向此手机号发送短信
//        SmsResponse smsResponse = SmsFactory.createSmsBlend(SupplierType.HUAWEI).sendMessage(mobile, code);

//        String result = SmsSender.getInstance().sendCaptcha(mobile, code);


        smsSender.sendCaptcha(mobile, code);
//        smsSender.sendNotify(mobile, null);

//        LinkedHashMap<String, String> map = SmsUtil.getNewMap();
//        map.put("code", code);
//
//        SmsResponse smsResponse2 = SmsFactory.createSmsBlend(SupplierType.HUAWEI).sendMessage(mobile, "97c6fcf9fd0149b39c0ed0e53480116f", map);

        redisUtil.set(GlobalConstant.CAPTCHA + StringPool.UNDERSCORE + mobile, code, 60L);
        return R.ok(Boolean.TRUE);
    }

    /**
     * 验证码
     */
    @PostMapping("/captcha")
    @XjrLog(value = "验证码登录")
    public R loginByCaptcha(@RequestBody LoginCaptchaDto loginCaptchaDto) {
        String mobile = loginCaptchaDto.getMobile();
        String code = redisUtil.get(GlobalConstant.CAPTCHA + StringPool.UNDERSCORE + mobile);
        if (code != null && StringUtils.equals(code, loginCaptchaDto.getCode())) {
            User user = userService.getOne(Wrappers.<User>lambdaQuery().eq(User::getMobile, mobile), false);
            if (user == null) {
                return R.error("用户不存在！");
            }
            if (user.getEnabledMark() == EnabledMark.DISABLED.getCode()) {
                return R.error("账户未启用");
            }
            //此登录接口登录web端
            StpUtil.login(user.getId(), "PC");

            SaSession tokenSession = StpUtil.getTokenSession();
            tokenSession.set(GlobalConstant.LOGIN_USER_INFO_KEY, user);

            Map<String, Object> vo = new HashMap<>(1);
            vo.put(GlobalConstant.TOKEN_KEY, StpUtil.getTokenValue());

            return R.ok(vo);
        }
        return R.error("验证码不存在！");
    }


    /**
     * 退出
     */
    @PostMapping("/logout")
    public R logout() {
        StpUtil.logout();
        return R.ok("登出成功！");
    }

    @PostMapping("/token")
    @ApiOperation(value = "根据keycloak-token 登录", notes = "传入keycloak-token")
    @XjrLog(value = "keycloak-token登录成功")
    public R loginByToken(@RequestBody KeyCloakLoginInfoDto dto) {

        Map<String, Object> credentialsMap = new HashMap<>(1);
        credentialsMap.put("secret", keyCloakConfig.getSecret());
        Configuration configuration = new Configuration(keyCloakConfig.getUrl(), keyCloakConfig.getRealm(), keyCloakConfig.getClientId(), credentialsMap, null);
        AuthzClient authzClient = AuthzClient.create(configuration);
        AccessTokenResponse response = authzClient.obtainAccessToken(keyCloakConfig.getUserName(), keyCloakConfig.getPassword());

        if (StrUtil.isNotBlank(response.getError())) {
            return R.error(response.getError());
        }
        //TODO keycloak 登陆过 解析token获取数据 做框架登录操作
        JWT jwt = JWTUtil.parseToken(dto.getToken());
        Object code = jwt.getPayload(keyCloakConfig.getPayload());

        User user = userService.getOne(Wrappers.lambdaQuery(User.class).eq(User::getCode, code));

        if (user == null) {
            return R.error("帐号密码错误！");
        } else if (!Integer.valueOf(1).equals(user.getEnabledMark())) {
            return R.error("该账号已被禁用！");
        }

        //此登录接口登录web端
        StpUtil.login(user.getId(), dto.getDevice());

        SaSession tokenSession = StpUtil.getTokenSession();
        tokenSession.set(GlobalConstant.LOGIN_USER_INFO_KEY, user);

        Map<String, Object> vo = new HashMap<>(1);
        vo.put(GlobalConstant.TOKEN_KEY, StpUtil.getTokenValue());

        return R.ok("登录成功！", vo);


    }
}
